# Trickier XBWT Tricks
This repository contains an implementation and a benchmark for XBWT construction,
which is, a succinct trie. The algorithms are described in

	Trickier XBWT Tricks

	by Enno Ohlebusch, Stefan Stauß, and Uwe Baier
	(hopefully to appear in SPIRE 2018)

## What is contained
This bundle of files consist of the following parts:
1. The algorithms required to construct and use the XBWT,
   contained in the `include`- and `lib`-directory
2. Data for reproducible computation results, see `rcrdata`-directory
3. A utility program for multi-pattern search
4. A benchmark script to test the construction algorithms

## Requirements
To compile the program(s), you need a modern c++11 ready compiler such as 
[gcc](https://gcc.gnu.org/) version 4.7 or newer.

## Prerequisites
The software uses the Succinct Data Structure Library [SDSL-LITE](https://github.com/simongog/sdsl-lite) by Simon Gog.
You have to specify the SDSL-LITE path with the "lib" and "include" folder by

```
export SDSLLITE="path/to/sdsllite/"
```

## Installation
Just call the command `make`. It should produce two executables:
- `trie_construct.x`: program to construct XBWT tries
- `mpsearch.x`: program for multi-pattern-search using an XBWT trie

## Usage
This bundle consists of 3 programs:
- `create_trie_input.sh`: used to generate an input file for XBWT construction 
  out of a newline-seperated file
- `trie_construct.x`: used to construct a trie from a (prepared) input file
- `mpsearch.x`: multi-pattern search
Just call any program without a parameter and follow the usage instructions.

##Getting Started
The following example commands show how to construct a trie from the strings hello and world
and use it for multi-pattern-search:

```
$ echo hello > test.txt
$ echo -n world >> test.txt
$ ./create_trie_input.sh test.txt trieinput.xin
$ ./trie_construct.x trieinput.xin trie.xbwt
$ echo helloworld | ./mpsearch.x trie.xbwt
```

A simpler way to do multi-pattern-search is given by the following:
```
$ echo helloworld | ./mpsearch.x hello world
```

##Benchmark
A benchmark is contained. To run it, configure `benchmark.config` as required and
run the benchmark by calling `make benchmark`. All timings are given in milliseconds,
all memory peaks in bytes.
If you want to store the results, try `make -s benchmark > result.dat` to store results in
`result.dat`.

##Reproducible Computation Results
To reproduce the computation results from the paper, call `make rcrbenchmark`.
It will automatically download and prepare the test data and execute the benchmark.
Note that the file `benchmark.config` gets overwritten during this procedure.
